import numpy as np
import pickle
import os
import sys
from Simple_Neural_Network import model


def get_working_directory():
    if getattr(sys, 'frozen', False):
        return os.path.dirname(sys.executable)
    else:
        return os.path.dirname(os.path.abspath(__file__))


#Funkcje do ładowania bazy danych CIFAR-10 z ich strony
def load_cifar10_batch(filename):
    with open(filename, 'rb') as f:
        batch = pickle.load(f, encoding='bytes')
        X = batch[b'data']
        Y = batch[b'labels']
        X = X.reshape(-1, 3, 32, 32).transpose(0, 2, 3, 1)
        return X, Y


def load_cifar10_data(data_dir):
    X_train = []
    Y_train = []
    for i in range(1, 6):
        filename = os.path.join(data_dir, f'data_batch_{i}')
        X_batch, Y_batch = load_cifar10_batch(filename)
        X_train.append(X_batch)
        Y_train += Y_batch
    X_train = np.concatenate(X_train)
    Y_train = np.array(Y_train)

    X_test, Y_test = load_cifar10_batch(os.path.join(data_dir, 'test_batch'))
    Y_test = np.array(Y_test)

    return X_train, Y_train, X_test, Y_test

def save_model(nn, filename):
    with open(filename, 'w') as f: 
        n_x = nn.hidden_layer[0].n_inputs #zapisujemy rozmiary warstw
        n_h = len(nn.hidden_layer) #zapisujemy rozmiary warstw ukrytych
        n_y = 1 #zapisujemy rozmiary warstw wyjsciowych
        f.write(f"{n_x} {n_h} {n_y}\n")
        for perceptron in nn.hidden_layer:#zapisujemy parametry warstwy ukrytej
            W = perceptron.W
            b = perceptron.b
            W_str = ' '.join(map(str, W))
            f.write(f"{W_str} {b}\n")
        #zapisujemy parametry warstwy wyjściowej
        W_output = nn.output_layer.W
        b_output = nn.output_layer.b
        W_output_str = ' '.join(map(str, W_output))
        f.write(f"{W_output_str} {b_output}\n")

if __name__ == '__main__':
    os.chdir(get_working_directory())

    #Ładowanie danych
    X_train_full, Y_train_full, X_test_full, Y_test_full = load_cifar10_data("./cifar-10-batches-py")

    #Wybieramy dwie klasy
    classes = [8, 3]  #Pierwsza klasa to klasa negatywna druga to klasa pozytywna (obecnie statki - 8, koteczki -3)
    train_filter = np.isin(Y_train_full, classes)#filtrujemy tylko dla klas
    test_filter = np.isin(Y_test_full, classes)

    #wybieramy te przefiltrowane przez klasy
    X_train = X_train_full[train_filter]
    Y_train = Y_train_full[train_filter]
    X_test = X_test_full[test_filter]
    Y_test = Y_test_full[test_filter]

    #przypisywanie etykiet: 1 dla klasy pozytywnej (drugiej klasy), 0 dla negatywnej (pierwszej klasy)
    Y_train = np.array([1 if y == classes[1] else 0 for y in Y_train])
    Y_test = np.array([1 if y == classes[1] else 0 for y in Y_test])

    #Dzielimy kolory przez 255 by bylu miedzy 0-1
    X_train =X_train/255.0
    X_test =X_test/255.0

    #Splaszczamy tablice na wektory
    X_train_flatten = X_train.reshape(X_train.shape[0], -1).T
    X_test_flatten = X_test.reshape(X_test.shape[0], -1).T

    Y_train = Y_train.reshape(1, -1)
    Y_test = Y_test.reshape(1, -1)



    d,nn = model(X_train_flatten, Y_train, X_test_flatten, Y_test,n_h=4, num_iterations=1000, learning_rate=0.01)
    save_model(nn, 'model_parameters.txt')
