import numpy as np  # Import biblioteki do obliczeń numerycznych
import copy         # Import biblioteki do kopiowania obiektów
import matplotlib.pyplot as plt  # Import biblioteki do tworzenia wykresów

def sigmoid(x):
    #funkcja sigmoid przeksztalca wartosci na wartosci z zakresu (0,1)
    s = 1 / (1 + np.exp(-x))
    return s

class Perceptron:
    def __init__(self, n_inputs,activation='sigmoid', learning_rate=1.2):
        #Klasa reprezentujaca jeden percepton
        #n_inputs - liczba wejsc perceptonu
        #learning_rate - tepo uczenia
        self.n_inputs = n_inputs
        self.learning_rate = learning_rate
        self.activation = activation
        
        self.W = np.random.randn(n_inputs, 1) * 0.01   #Wagi perceptronu pomonozone za mala liczbe nie moze byc zero bo perceptrony musz byc rozne dlatego losujemy 
        self.b = np.zeros((1, 1)) #Bias perceptronu

    def forward(self, X):
        #propagacja w przod dla perceptoronu
        #X- dane wejsciowe macerz o wymiarach (n_inputs,m)
        # A - wyjscie perceptrony

        self.X = X  #Zapamiętanie danych wejściowych
        self.Z = np.dot(self.W.T, X) + self.b  #Wazymy dane wejsciowe przez wagi w perceptronie
        #liczmy prawdopodobienstwo na podstawie zwazonych wartosci
        if self.activation == 'sigmoid':
            self.A = sigmoid(self.Z)
        elif self.activation == 'tanh':
            self.A = np.tanh(self.Z)
        return self.A

    def backward(self, dA):
        #propagacja wsteczna dla perceptronu
        #da - pochodna kosztu wujscia perceptronu
        #dX -pochodna kosztu wzgledem wejscia percetronu

        m = self.X.shape[1]#liczba przykładów (probek) w zbiorze danych
        
        if self.activation == 'sigmoid':
            dZ = dA * self.A * (1 - self.A) 
        elif self.activation == 'tanh':
            dZ = dA * (1 - self.A ** 2)
        self.dW = (1 / m) * np.dot(self.X, dZ.T)#pochodna dw 
        self.db = (1 / m) * np.sum(dZ, axis=1, keepdims=True)# pochodna dwb
        dX = np.dot(self.W, dZ)
        return dX

    def update_parameters(self):
        #aktualizacja parametrów perceptronu
        self.W -= self.learning_rate * self.dW
        self.b -= self.learning_rate * self.db

class Neural_Network:
    def __init__(self, n_x, n_h, n_y, learning_rate=1.2):
        #Klasa do reprezacji dwuwarstowej sieci neuronowej z wykosztysniem klasy perceptronu wyzej
        #n_x-liczba cech w warwtie wejsciowe
        #n_h - liczba neuronow w warstwie ukrytej
        self.learning_rate = learning_rate

        #Tworzenie warstwy ukrytej z n_h perceptronami
        self.hidden_layer = [Perceptron(n_inputs=n_x, activation='tanh', learning_rate=learning_rate) for _ in range(n_h)]

        #Tworzenie warstwy wyjściowej z jednym perceptronem
        self.output_layer = Perceptron(n_inputs=n_h, activation='sigmoid', learning_rate=learning_rate)

    def forward_propagation(self, X):
        #funkcja propagacji w przod dla calej sieci neuronowej


        #Propagacja w przód przez warstwę ukrytą
        self.A1 = []
        for perceptron in self.hidden_layer:
            A = perceptron.forward(X)
            self.A1.append(A)
        self.A1 = np.vstack(self.A1)  #Połączenie  wyjść wszystkich perceptronów w warstwie ukrytej w jedna tablive

        #Propagacja w przód przez warstwę wyjściową
        A2 = self.output_layer.forward(self.A1)
        return A2

    def compute_cost(self, A2, Y):
        #FUnkcja do obliczenia funkcji kosztu dla modelu

        m = Y.shape[1]  #liczba przykładów

        #obliczenie kosztu za pomocą funkcji logarytmicznej (log-loss)
        logprobs =np.multiply(Y, np.log(A2)) + np.multiply(1 - Y, np.log(1 - A2))
        cost = -(1/m)*np.sum(logprobs)

        cost = float(np.squeeze(cost))
        return cost

    def backward_propagation(self, X, Y):
        #FUnkcja propagacji wtecznje
        m = X.shape[1]  #Liczba przykładów

        #propagacja wstecz przez warstwę wyjściową
        dA2 = - (np.divide(Y, self.output_layer.A) - np.divide(1 - Y, 1 - self.output_layer.A))
        dA1 = self.output_layer.backward(dA2)

        # Propagacja wstecz przez warstwę ukrytą
        dX = np.zeros_like(X)
        for i, perceptron in enumerate(self.hidden_layer):
            dA_prev = perceptron.backward(dA1[i, :].reshape(1, m))
            dX += dA_prev  #sumujemy gradienty z każdego perceptronu

    def update_parameters(self):
        # Aktualizacja parametrów w warstwie ukrytej
        for perceptron in self.hidden_layer:
            perceptron.update_parameters()

        # Aktualizacja parametrów w warstwie wyjściowej
        self.output_layer.update_parameters()

    def train(self, X, Y, num_iterations=1000):
        #funkcja trenujaca siec neuronowa
        for i in range(num_iterations):
            #popagacja w przód
            A2 = self.forward_propagation(X)

            #obliczenie kosztu
            cost = self.compute_cost(A2, Y)

            #propagacja wstecz
            self.backward_propagation(X, Y)

            #aktualizacja parametrów
            self.update_parameters()

            #wypisanie kosztu co 1000 iteracji
            print(f"Koszt po iteracji {i}: {cost}")

    def predict(self, X):
        #funkcja przewidujaca prawdopodobienstwo
        A2 = self.forward_propagation(X)
        predictions = (A2 > 0.5).astype(int)
        return predictions

def model(X_train, Y_train, X_test, Y_test, n_h, num_iterations=2000, learning_rate=0.5):
    #budowanie i trenowanie modelu sieci neuronowej
    #przyjmuje ilosc neuronowj w wartwie ukrytej dane treninogowe i testowe 

    n_x = X_train.shape[0]  #liczba cech wejściowych
    n_y = Y_train.shape[0]  #liczba neuronów w warstwie wyjściowej 
    
    #inicjalizacja sieci neuronowej
    nn = Neural_Network(n_x=n_x, n_h=n_h, n_y=n_y, learning_rate=learning_rate)
    
    #trenowanie modelu
    nn.train(X_train, Y_train, num_iterations=num_iterations)
    
    #przewidywanie wyników na zbiorach treningowych i testowych
    Y_prediction_train = nn.predict(X_train)
    Y_prediction_test = nn.predict(X_test)
    
    #obliczenie dokładności
    train_accuracy = 100 - np.mean(np.abs(Y_prediction_train - Y_train)) * 100
    test_accuracy = 100 - np.mean(np.abs(Y_prediction_test - Y_test)) * 100
    
    print(f"Train dokladnosc: {train_accuracy} %")
    print(f"Test dokladnosc: {test_accuracy} %")
    
    #zwrot wyników
    d = {
        "Y_prediction_test": Y_prediction_test,
        "Y_prediction_train": Y_prediction_train,
        "train_accuracy": train_accuracy,
        "test_accuracy": test_accuracy,
        "learning_rate": learning_rate,
        "num_iterations": num_iterations
    }
    
    return d,nn