import numpy as np
import matplotlib.pyplot as plt
from Simple_Neural_Network import Neural_Network
from Create import load_cifar10_data
import os
import sys

def get_working_directory():
    if getattr(sys, 'frozen', False):
        return os.path.dirname(sys.executable)
    else:
        return os.path.dirname(os.path.abspath(__file__))

def predict_image_nn(image, nn):
    image_flatten = image.reshape(-1, 1)
    prediction = nn.predict(image_flatten)
    return int(prediction[0, 0])

def load_model(filename):
    with open(filename, 'r') as f:
        lines = f.readlines()

    #Wczytujemy rozmiary warstw
    n_x, n_h, n_y = map(int, lines[0].strip().split())

    #Inicjalizujemy sieć neuronową z wczytanymi rozmiarami warstw
    nn = Neural_Network(n_x=n_x, n_h=n_h, n_y=n_y)

    idx = 1
    #Wczytujemy parametry warstwy ukrytej
    for i in range(n_h):
        line = lines[idx].strip()
        line = line.replace('[', '').replace(']', '')
        params = list(map(float, line.split()))
        W_size = nn.hidden_layer[i].n_inputs
        W = np.array(params[:-1]).reshape(W_size, 1)
        b = np.array([params[-1]]).reshape(1, 1)
        nn.hidden_layer[i].W = W
        nn.hidden_layer[i].b = b
        idx += 1

    #Wczytujemy parametry warstwy wyjściowej
    line = lines[idx].strip()
    line = line.replace('[', '').replace(']', '')
    params = list(map(float, line.split()))
    W_size = nn.output_layer.n_inputs
    W = np.array(params[:-1]).reshape(W_size, 1)
    b = np.array([params[-1]]).reshape(1, 1)
    nn.output_layer.W = W
    nn.output_layer.b = b

    return nn

if __name__ == '__main__':
    os.chdir(get_working_directory())

    #Ładowanie danych testowych z pliku ciraf
    data_dir = os.path.join(get_working_directory(), "cifar-10-batches-py")
    #ladujemy tylko testowe
    _, _, X_test_full, Y_test_full = load_cifar10_data(data_dir)
    
    ##############

    #filtrujmey przez klase pierwsza (neg) i druga (poz)
    classes = [8, 3]
    test_filter = np.isin(Y_test_full, classes)
    X_test = X_test_full[test_filter]
    Y_test = np.array([1 if y == classes[1] else 0 for y in Y_test_full[test_filter]])

    #Dla NN
    X_test = X_test / 255.0

    nn = load_model('model_parameters.txt')
    
    X_test_flatten = X_test.reshape(X_test.shape[0], -1).T
    Y_pred = nn.predict(X_test_flatten)
    test_accuracy = 100 - np.mean(np.abs(Y_pred - Y_test)) * 100
    print(f"Dokładność na zbiorze testowym: {test_accuracy:.2f}%")

    # Podajemy zakres indeksów
    num_samples = X_test.shape[0]
    print(f"Wybierz indeks obrazu do przetestowania (0 - {num_samples - 1}):")
    index = int(input())
    #Losowanie obrazka z dwóch  klas ktore ustalilismy
    #index = np.random.randint(len(X_test))
    image = X_test[index]
    true_class = Y_test[index]

    prediction =predict_image_nn(image,nn) 
    print(f"Predykcja: {'Pozytywna' if prediction == 1 else 'Negatywna'}")
    print(f"Prawdziwa etykieta: {'Pozytywna' if true_class == 1 else 'Negatywna'}")

    #Wizualizacja
    plt.imshow(image)
    #pozytywna dla prediction 1
    plt.title(f"Predykcja: {'Pozytywna' if prediction == 1 else 'Negatywna'}, Prawdziwa: {'Pozytywna' if true_class == 1 else 'Negatywna'}")
    plt.axis('off')
    plt.show()
